import pygame
import toolbox

class HUD():
    def __init__(self, screen, player):
        self.screen = screen
        self.player = player

        self.state = 'mainmenu'
        
            # Type these commands into python shell for list of fonts:
                #   import pygame
                #   pygame.font.get_fonts()
            # A kind of cool fonts: sfnssymbolsregularotf (it's all rectangles), couriernewbolditalicttf,
            #   trebuchetmsttf, noteworthyttc
        self.hud_font = pygame.font.SysFont("couriernewboldttf", 30)
        self.hud_font_tutorial = pygame.font.SysFont("default", 30)
        self.hud_font_med = pygame.font.SysFont("default", 50)
        self.hud_font_big = pygame.font.SysFont("default", 100) #And-E did 80 for the big font
        self.hud_font_credits = pygame.font.SysFont("default", 15)
         
        self.score_text = self.hud_font.render("WB WINS", True, (255, 255, 255))#boolean in list is anti-aliasing

        # Load stuff for the main menu
        self.title_image = pygame.image.load("../assets/title.png")
        self.start_blink_timer_max = 80
        self.start_blink_timer = self.start_blink_timer_max
        self.start_text = self.hud_font.render("Press any key to begin", True, (255, 255, 255))
        self.tutorial_text = self.hud_font_tutorial.render("WASD to move - CLICK to shoot - SPACE to place crate - RIGHT CLICK to place explosive crate", True, (255,255,255))
        self.credits_text = self.hud_font_credits.render("", True, (0,0,0))

        # Load stuff for the game over screen
        self.game_over_text = self.hud_font_big.render("Game Over", True, (0, 0, 0))
        self.reset_button = pygame.image.load("../assets/BtnReset.png")
        
        # Load the images for the ammo tiles
        self.crate_icon = pygame.image.load("../assets/Crate.png")
        self.explosive_crate_icon = pygame.image.load("../assets/ExplosiveBarrel.png")
        self.split_shot_icon = pygame.image.load("../assets/iconSplitPurple.png")
        self.stream_shot_icon = pygame.image.load("../assets/iconStream.png")
        self.burst_shot_icon = pygame.image.load("../assets/iconBurst.png")
        self.normal_shot_icon = pygame.image.load("../assets/BalloonSmall.png")
        self.rocket_shot_icon = pygame.image.load("../assets/BalloonRocket.png")
        self.bouncy_shot_icon = pygame.image.load("../assets/iconBalloonSmallBouncy.png")

        # make all the ammo tiles
        self.crate_ammo_tile = AmmoTile(self.screen, self.crate_icon, self.hud_font)
        self.explosive_crate_ammo_tile = AmmoTile(self.screen, self.explosive_crate_icon, self.hud_font)
        self.balloon_ammo_tile = AmmoTile(self.screen, self.normal_shot_icon, self.hud_font)
        
    def update(self):
        if self.state == 'ingame':
            # Draw the score text
            self.score_text = self.hud_font.render("Score: " + str(self.player.score), True, (255, 255, 255))
            self.screen.blit(self.score_text, (10, 10))

            # Draw the ammo tiles
            tile_x = 392 # Use 392 to make tiles in the center of the screen
            self.crate_ammo_tile.update(tile_x, self.screen.get_height(), self.player.crate_ammo)
            tile_x += self.crate_ammo_tile.width
            self.explosive_crate_ammo_tile.update(tile_x, self.screen.get_height(), self.player.explosive_crate_ammo)
            tile_x += self.crate_ammo_tile.width
            # Figure out which icon to use for special ammo
            if self.player.shot_type == 'normal':
                self.balloon_ammo_tile.icon = self.normal_shot_icon
            elif self.player.shot_type == 'split':
                self.balloon_ammo_tile.icon = self.split_shot_icon
            elif self.player.shot_type == 'burst':
                self.balloon_ammo_tile.icon = self.burst_shot_icon
            elif self.player.shot_type == 'stream':
                self.balloon_ammo_tile.icon = self.stream_shot_icon
            elif self.player.shot_type == 'rocket':
                self.balloon_ammo_tile.icon = self.rocket_shot_icon
            elif self.player.shot_type == 'bouncy':
                self.balloon_ammo_tile.icon = self.bouncy_shot_icon
            
            self.balloon_ammo_tile.update(tile_x, self.screen.get_height(), self.player.special_ammo)

        elif self.state == 'mainmenu':
            self.start_blink_timer -= 1
            if self.start_blink_timer <= 0:
                self.start_blink_timer = self.start_blink_timer_max
        # title image
            title_x, title_y = toolbox.centeringCoords(self.title_image, self.screen)
            title_y -= 40
            self.screen.blit(self.title_image, (title_x, title_y))
            text_x, text_y = toolbox.centeringCoords(self.start_text, self.screen)
            text_y += 110
            if self.start_blink_timer > 30:
                self.screen.blit(self.start_text, (text_x, text_y))
        # Tutorial text
            text_x, text_y = toolbox.centeringCoords(self.tutorial_text, self.screen)
            text_y = self.screen.get_height() - 50
            self.screen.blit(self.tutorial_text, (text_x, text_y))
            # Credits text
#            text_x, text_y = toolbox.centeringCoords(self.credits_text, self.screen)
#            text_y += 250
#            self.screen.blit(self.credits_text, (text_x, text_y))

        elif self.state == 'gameover':
            text_x, text_y = toolbox.centeringCoords(self.game_over_text, self.screen)
            text_y -= 60
            self.screen.blit(self.game_over_text, (text_x, text_y))
            self.score_text = self.hud_font.render("Final Score: " + str(self.player.score), True, (255, 255, 255))
            text_x, text_y = toolbox.centeringCoords(self.score_text, self.screen)
            text_y += 0
            self.screen.blit(self.score_text, (text_x, text_y))
            button_x, button_y = toolbox.centeringCoords(self.reset_button, self.screen)
            button_y += 100
            button_rect = self.screen.blit(self.reset_button, (button_x, button_y))

            events = pygame.event.get()
            for event in events:
                if event.type == pygame.MOUSEBUTTONDOWN:
                    mouse_position = pygame.mouse.get_pos()
                    if button_rect.collidepoint(mouse_position):
                        self.state = 'mainmenu'

class AmmoTile():
    def __init__(self, screen, icon, font):
        self.screen = screen
        self.icon = icon
        self.font = font
        self.bg_image = pygame.image.load("../assets/hudTile.png")
        self.width = self.bg_image.get_width()

    def update(self, x, y, ammo):
        # Draw tile background
        tile_rect = self.bg_image.get_rect()
        tile_rect.bottomleft = (x, y)
        self.screen.blit(self.bg_image, tile_rect)
        # Draw icon
        icon_rect = self.icon.get_rect()
        icon_rect.center = tile_rect.center
        self.screen.blit(self.icon, icon_rect)
        # Draw ammo number/amount
        ammo_text = self.font.render(str(ammo), True, (255, 255, 255))
        self.screen.blit(ammo_text, tile_rect.topleft)
